# Upgrading from V5 to V6

This is a long overdue refactor of the entire library. As feature requests have come in, the code and test suite had become increasingly spaghettified, and this new version is a massive improvement on the previous implementation.

For the most common use cases, and the basic mocking and inspecting api, very little has changed. The main changes are around how fetch-mock is configured and installed, and turning a few nice new features on by default (couldn't do so in v5 as would've been a breaking change)

Feedback gratefully received when in beta - please raise an issue :-)

To try it out:
```
npm install fetch-mock@6.0.0-beta.2
```
(Bear in mind that the docs may still include a few inaccuracies)

# Changes

## Configuration and installation
- *No change to mocking fetch as a global*
- Uses whatever version of `node-fetch` you have already installed in your project i.e. can be used with `node-fetch` 2
- `.sandbox()` no longer accepts a custom `Promise` implementation as a parameter
- Sandboxes can now be 'sub-classed' i.e. define some mocks on a sandbox, then call `.sandbox()` on it to create a new one inheriting all its settings
- No longer has a `setImplementations()` method. Instead, assign custom `Promise` and `fetch` helper classes  directly to the `config` property of the fetch-mock instance. This may be done before or afeter sandboxing
- No longer has a `configure()` method. Instead set options directly on the `.config` property. See the api docs for a complete list of options

## Routing
- `^` is no longer a valid way of matching the beginnig of strings; use `begin:` instead
- `overrideRoutes` option allows for existing routes in a mock to be overwritten. It's also possible to define multiple routes with 'the same' matcher. Default behaviour is to error

## Responses
- `Content-Length` header generated by default for all responses (can be configured to not do so globally or per response.)
- Objects will be converted to `JSON` responses if they contain any properties not listed in the docs as special response config options (previous behaviour was to convert if any of the properties were present). This mean that e.g. `{user: 'me', status: 'happy'}` can easily be sent as `JSON`
- `__redirectUrl` option is now named `redirectUrl`
- `times` option is now named `repeat`

## Inspecting
- All inspection methods now filter, in addition to the existing check against `matcher.toString()`, based on the following rules
    + An exact url
    + `true` for matched calls only
    + `false` for unmatched calls only
    + `undefined` includes all calls to `fetch` in order
- When `fetch` was last called with a `Request` object, `lastUrl()` and `lastOpts()` give easier access to the `url` and full `Request` object